#!/usr/bin/env python
# -*- coding: latin-1 -*-

# Imports
from networkhandler import *
from balls import Ball
from bats import Bat
from prediction import *
from replicators import *
from pygameutils import *
import time, math

class Server:
	""" The server of the game """
	REPLICATOR = 0
	PREDICTOR = 1
	PLAYERRECT = 2

	def __init__(self, listenPort):
		""" Inits a server with its port it will listen to traffic,
			sets up ball, replicator """
		# Network
		self.port = listenPort
		self.network = NetworkServer(self, self.port)
		self.communicator = SettingCommunicator(self, self.port)
		self.network.startListening()
		self.communicator.startListening()
		
		# Ball
		self.ballReplicator = BallReplicator(self)
		self.ball = Ball((400, 150), math.pi/4, 300)
		
		# Bats/paddles
		self.players = {}
		self.frameRate = 60
		self.scores = [0, 0]

	def setFramerate(self, frameRate):
		""" Set the framerate of the server """
		self.frameRate = frameRate
		print 'Changing server framerate to', self.frameRate
		return self.frameRate

	def addPlayer(self, addr):
		""" Add a new player to the game """
		if len(self.players) == 0:
			self.players[addr] = [BatReplicator(self),  BatPredictor((560, 130)), \
					Bat((560, 130), 0, 1)]
		elif len(self.players) == 1:
			self.players[addr] = [BatReplicator(self),  BatPredictor((60,130)), \
					Bat((60, 130), 0, 2)]
		# We must store the senders ip/port to be able to decide
		# who packets are from, when receiveing new data from network
		self.players[addr][0].setSender(addr)

	def update(self):
		""" Updates the state of the server """
		timeNow = time.time()
		self.updateReplicators()
		self.doCollisionCheck()
		reactor.doSelect(0)
		timeNow += (1.0 / self.frameRate)
		diff = timeNow - time.time()
		if diff > 0:
			time.sleep(diff)

	def updateReplicators(self):
		""" Updates the servers replicators """
		scoreValue = self.ball.updatePosition()
		# Check if someone scored or it just was an ordinary bounce
		if scoreValue in [1, 2]:
			self.scores[scoreValue-1] = self.scores[scoreValue-1] + 1
			self.ballReplicator.setScore(scoreValue, self.scores[scoreValue-1])
		else:
			self.ballReplicator.setPosition(self.ball.getPosition())

		# Get new position for all players, and set that position to the
		# correct replicator
		for player in self.players.keys():
			replicator = self.players[player][self.REPLICATOR]
			predictor = self.players[player][self.PREDICTOR]
			playerBat = self.players[player][self.PLAYERRECT]
			# Get the old position
			newPos = predictor.getPosition()
			playerBat.move(newPos)
			replicator.setPosition(newPos)

	def doCollisionCheck(self):
		""" Does collision detection between objects """
		self.ball.doCollisionCheck(self.players)
